package io.github.rmuhamedgaliev.service.process.impl;

import io.github.rmuhamedgaliev.service.process.ProcessInformation;
import org.springframework.stereotype.Service;
import io.github.rmuhamedgaliev.model.process.LocalProcess;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Developer: Muhamedgaliev Rinat
 */
@Service(value = "processInformation")
public class ProcessInformationImpl implements ProcessInformation
{

    public List<LocalProcess> getProcessList()
    {
        List<LocalProcess> processes = new ArrayList<>();
        try
        {
            Process process = Runtime.getRuntime().exec("ps -eo comm,euser,stat,vsz,rss,%cpu,cputime,start_time,pid,%mem,ppid,args");
            BufferedReader outPutCommand = new BufferedReader(new InputStreamReader(process.getInputStream(), "UTF-8"));
            String output = null;
            outPutCommand.readLine();
            while ((output = outPutCommand.readLine()) != null)
            {
                LocalProcess localProcess = null;
                StringBuffer commandArgs = new StringBuffer();
                StringTokenizer clearCommandOutput = new StringTokenizer(output, " ");
                String command = clearCommandOutput.nextToken();
                String user = clearCommandOutput.nextToken();
                String stat = clearCommandOutput.nextToken();
                String vsz = clearCommandOutput.nextToken();
                String rss = clearCommandOutput.nextToken();
                String cpu = clearCommandOutput.nextToken();
                String cpuTime = clearCommandOutput.nextToken();
                String startTime = clearCommandOutput.nextToken();
                String pid = clearCommandOutput.nextToken();
                String memory = clearCommandOutput.nextToken();
                String ppid = clearCommandOutput.nextToken();
                String commandWithArgs;
                while (clearCommandOutput.hasMoreElements())
                {
                    commandArgs.append(" ").append(clearCommandOutput.nextElement());
                }
                commandWithArgs = commandArgs.toString();
                localProcess = new LocalProcess(command, user, stat, vsz, rss, cpu, cpuTime, startTime, pid, memory, ppid, commandWithArgs);
                processes.add(localProcess);
            }
            outPutCommand.close();
            return processes;
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return processes;
    }

    @Override
    public double getLoadAverage()
    {
        OperatingSystemMXBean osBean = ManagementFactory.getOperatingSystemMXBean();
        return osBean.getSystemLoadAverage();
    }

    @Override
    public List<LocalProcess> getZombieProcessesList()
    {
        List<LocalProcess> zombiesProcesses = new ArrayList<>();
        for (LocalProcess localProcess : getProcessList())
        {
            if (localProcess.getStat().toUpperCase().contains("Z"))
            {
                zombiesProcesses.add(localProcess);
            }
        }
        return zombiesProcesses;
    }

    @Override
    public List<LocalProcess> findProcessByCommandOrPID(String findParam)
    {
        List<LocalProcess> allProcesses = getProcessList();
        List<LocalProcess> requredProcesses = new ArrayList<>();
        if (findParam.trim().length() != 0)
        {
            for (LocalProcess localProcess : allProcesses)
            {
                if (localProcess.getCommand().contains(findParam)
                        || localProcess.getCommandWithArgs().contains(findParam)
                        || localProcess.getPid().contains(findParam))
                {
                    requredProcesses.add(localProcess);
                }
            }
        }
        return requredProcesses;
    }
}
